// Returns whether the weapon should be properly picked up.
//  Side effects: gives you the weapon, if running over it would pick it up.
function int Dakka_PickupWeapon(int wep, int dropped, int shouldstay)
{
    int item = DakkaKnownGuns[wep][0];
    int hasgun = CheckInventory(item) > 0;

    int wepsStay = !!GetCVar("sv_weaponstay");
    int stays = (wepsStay && shouldstay) && !dropped;
    if (stays && hasgun) { return 0; }

    int a1c_start = 0, a1c_end = 0, a1cap = 0;
    int a2c_start = 0, a2c_end = 0, a2cap = 0;

    int ammo1 = DakkaKnownGuns[wep][1];
    int ammo2 = DakkaKnownGuns[wep][2];

    int hasa1  = !!StrLen(ammo1);
    int hasa2  = !!StrLen(ammo2);

    if (hasa1)
    {
        a1c_start = CheckInventory(ammo1);
        a1cap     = GetAmmoCapacity(ammo1);
        if (dropped) { SetAmmoCapacity(ammo1, a1cap * 2); }
    }

    if (hasa2)
    {
        a2c_start = CheckInventory(ammo2);
        a2cap     = GetAmmoCapacity(ammo2);
        if (dropped) { SetAmmoCapacity(ammo2, a2cap * 2); }
    }

    GiveInventory(item, 1);

    int pickupDidSomething = !hasgun;
    int changed;

    if (hasa1)
    {
        if (dropped)
        {
            a1c_end = CheckInventory(ammo1);
            
            // cut off either half the ammo gained, or cap it at the actual max
            int a1diff = max(((a1c_end - a1c_start) + 1) / 2, a1c_end - a1cap);
            TakeInventory(ammo1, a1diff);
            SetAmmoCapacity(ammo1, a1cap);
        }

        // refresh since it could've changed
        a1c_end = CheckInventory(ammo1);
        changed = (a1c_end != a1c_start);
        pickupDidSomething |= changed;
    }

    if (hasa2)
    {
        if (dropped)
        {
            a2c_end = CheckInventory(ammo2);
            int a2diff = max(((a2c_end - a2c_start) + 1) / 2, a2c_end - a2cap);
            TakeInventory(ammo2, a2diff);
            SetAmmoCapacity(ammo2, a2cap);
        }

        a2c_end = CheckInventory(ammo2);
        changed = (a2c_end != a2c_start);
        pickupDidSomething |= changed;
    }

    if (pickupDidSomething && !hasGun)
    {
        int doswitch = 0;

        switch (CtoS_SwitchOnPickup[PlayerNumber()])
        {
          case 0:
            break;

          case 1:
            int oldPriority;
            int newPriority = DakkaGunPriorities[wep];
            int curGun = getCurrentWeaponIndex();

            if (curGun == -1) { oldPriority = 0x7FFFFFFF; }
            else { oldPriority = DakkaGunPriorities[curGun]; }

            if (oldPriority >= newPriority) { doswitch = 1; }
            break;
          
          case 2:
            doswitch = 1;
            break;
        }

        if (doswitch) { SetWeapon(item); }
    }

    return pickupDidSomething;
}

function int dak_armorscore(int ratio, int points, int mode)
{
    switch (mode)
    {
        case AMODE_HEALTH: return ratio * points;
        case AMODE_ARMOR:  return points;
    }

    return points;
}

// Ditto, but s/weapon/armor/g.
function int Dakka_PickupArmor(int whichArmor, int count)
{
    int armor       = DakkaKnownArmors[whichArmor][0];
    int armorMode   = DakkaArmorValues[whichArmor][ARM_COMPAREMODE];
    int armorType   = DakkaArmorValues[whichArmor][ARM_TYPE];
    int armorGive   = DakkaArmorValues[whichArmor][ARM_PICKUPPOINTS];
    int armorMax    = DakkaArmorValues[whichArmor][ARM_MAXPOINTS];
    int armorRatio  = DakkaArmorValues[whichArmor][ARM_PROTECTION];

    if (armorGive == 0) { return 0; }

    int fromIndex   = getCurrentArmorIndex();
    int fromPoints  = CheckInventory("Armor");
    int fromArmor, fromRatio, fromScore;

    // If we don't know what armor he has, assume he has none since no other
    //  assumption would be sensible.
    
    if (fromIndex == -1)
    {
        fromArmor = "";
        fromRatio = 1;
        fromScore = 0;
    }
    else
    {
        fromArmor = DakkaKnownArmors[fromIndex][0];
        fromRatio = DakkaArmorValues[fromIndex][ARM_PROTECTION];
        fromScore = dak_armorscore(fromRatio, fromPoints, armorMode);
    }

    int toPoints = min(fromPoints + (count * armorGive), armorMax);
    int toScore;
    
    switch (armorType)
    {
        case ATYPE_BONUS: toScore = dak_armorscore(fromRatio, toPoints, armorMode); break;
        default: toScore = dak_armorscore(armorRatio, toPoints, armorMode); break;
    }

    /*Print(s:"from: ", f:fromScore, s:"  -  to: ", f:toScore,
        s:"\ngiving \"", s:armor, s:"\", ", d:toPoints, s:" at ratio ", f:armorRatio,
        s:"\ncount is ", d:count);*/

    // If we don't actually gain anything from picking up this armor, abort.
    if (fromScore >= toScore) { return 0; }

    int endArmor;

    /*
     * If the player has no known armor at this point, we assume he has no armor.
     *  The armor he will end up with is the armor being given, without question.
     *
     * If the player has known armor, and this pickup is considered a bonus,
     *  the ending armor type is the current armor.
     *
     * Otherwise, since we've determined that we're gaining something from this
     *  new armor, the end armor is the armor we're giving.
     */
    
    if (fromIndex == -1) { endArmor = armor; }
    else switch (armorType)
    {
        case ATYPE_BONUS: endArmor = fromArmor; break;
        default:          endArmor = armor; break;
    }

    // We now know what armor value we're ending at, and what armor type it is.
    
    if (!strcmp(endArmor, fromArmor))
    {
        GiveInventory("Dakka_OneArmor", toPoints - fromPoints);
    }
    else
    {
        TakeInventory("BasicArmor", 0x7FFFFFFF);
        GiveInventory(endArmor, 1);

        int curArmor = CheckInventory("Armor");

        if (curArmor < toPoints)
        {
            GiveInventory("Dakka_OneArmor", toPoints - curArmor);
        }
        else
        {
            TakeInventory("BasicArmor", curArmor - toPoints);
        }
    }

    return 1;
}

// Returns 0 if no pickup was performed, 1 if it was performed, and 2 if a
//  pickup was performed and a low message should be displayed.
// Side effects: gives you the health, dummy.
function int Dakka_PickupHealth(int index, int amount)
{
    int fromHealth  = GetActorProperty(0, APROP_Health);
    int maxHealth   = getMaxHealth();

    int health      = DakkaHealthValues[index][HEAL_PICKUPPOINTS];
    int giveMax     = DakkaHealthValues[index][HEAL_MAXPOINTS];
    int giveMaxMode = DakkaHealthValues[index][HEAL_MAXMODE];
    int giveLow     = DakkaHealthValues[index][HEAL_LOWPOINTS];
    int giveLowMode = DakkaHealthValues[index][HEAL_LOWMODE];

    switch (giveMaxMode)
    {
      default: break;
      case HMAX_ADDITIVE:
        giveMax += maxHealth;
        break;

      case HMAX_SCALED:
        giveMax = FixedMul(maxHealth, giveMax);
        break;
    }

    switch (giveLowMode)
    {
      default: break;
      case HLOW_ADDITIVE:
        giveLow += maxHealth;
        break;

      case HLOW_SCALED:
        giveLow = FixedMul(maxHealth, giveLow);
        break;
    }
    
    int toHealth = min(giveMax, fromHealth + (health * amount));
    int isLow = fromHealth <= giveLow;

    if (fromHealth >= toHealth) { return 0; }
    SetActorProperty(0, APROP_Health, toHealth);

    if (isLow) { return 2; }
    return 1;
}



script 649 (int index, int dropped, int weaponstay)
{
    if (!IsServer) { terminate; }

    int classslot = dakkaClassNum() + 1;
    if (index < 0 || index >= PICKUPCOUNT) { SetResultValue(0); terminate; }
    int alwaysPickup = Dakka_ForcePickups[classslot][index];

    int wepsStay = !!GetCVar("sv_weaponstay");
    int doPickup = 1;
    int didPickup = 0;

    int ammoMult = 1.0;
    if (dropped) { ammoMult /= 2; }
    if (GetCVar("sv_doubleammo")) { ammoMult *= 2; }

    if (!(doPickup || alwaysPickup))
    {
        SetResultValue(0);
        terminate;
    }

    int i;
    int lowHealthMsg = -1;
    int hadWep = 0;

    for (i = 0; i < PICKUPSLOTCOUNT; i++)
    {
        int item        = DakkaPickups[classslot][index][i];
        int count       = DakkaPickupCounts[classslot][index][i];
        int countinv    = DakkaPickup_GoesIntoInventory[classslot][index][i];
        int ammo        = isAmmo(item);
        int wep         = getKnownWeaponIndex(item);
        int armor       = getKnownArmorIndex(item);
        int health      = getKnownHealthIndex(item);

        if (StrLen(item) == 0 || count < 0) { continue; }
        if (count == 0) { count = 1; }

        if (ammo) { count = FixedMul(ammoMult, count); }

        //Log(s:"Would get ", d:count, s:" of ", s:item, s:" (is ", s:cond(ammo, "", "not "),
        //    s:"ammo); would ", s:cond(countinv, "", "not "), s:"count inventory");

        if (wep != -1)
        {
            hadWep = 1;
            int gotgun = Dakka_PickupWeapon(wep, dropped, weaponstay);
            didPickup |= gotgun;
        }
        else if (armor != -1)
        {
            int gotarm = Dakka_PickupArmor(armor, count);
            didPickup |= gotarm;
        }
        else if (health != -1)
        {
            int gotheal = Dakka_PickupHealth(health, count);
            didPickup |= !!gotheal;
            if (gotheal == 2) { lowHealthMsg = health; }
        }
        else
        {
            int startcount = CheckInventory(item);
            GiveAmmo(item, count);
            int endcount   = CheckInventory(item);

            if (countinv) { didPickup |= (startcount < endCount); }
            else { didPickup = 1; }
        }
    }

    didPickup |= alwaysPickup;

    if (didPickup)
    {
        int flags;

        flags |= lowHealthMsg != -1;
        flags |= (!!dropped) << 1;

        // This will break if more than 65536 pickups get defined.
        //  If that happens, though, we have other issues.
        if (flags & 1)
        {
            ACS_ExecuteAlways(547, 0, PlayerNumber(), packShorts(lowHealthMsg, index), flags);
        }
        else
        {
            ACS_ExecuteAlways(547, 0, PlayerNumber(), index, flags);
        }
    }

    SetResultValue(didPickup);
    if (weaponstay && wepsStay && !dropped && hadWep) { SetResultValue(0); }
}

script 547 (int pln, int index, int flags) clientside
{
    int classslot   = dakkaClassNum() + 1;
    int message;
    int silent      = !!(GetCvar("compat_silentpickup"));
    int myPln       = PlayerNumber();

    // Low health
    if (flags & 1)
    {
        message = chooseLowHealthMessage(leftShort(index), !(flags & 2));
        index = rightShort(index);
    }
    else { message = chooseMessage(classslot, index, !(flags & 2)); }

    if (strstr(message, "\\$") == 0)
    {
        message = sliceString(message, 2, StrLen(message));
    }
    else if (GetChar(message, 0) == '$')
    {
        message = StrParam(l:sliceString(message, 1, StrLen(message)));
    }

    int pkupSound   = DakkaPickupSounds[classslot][index];

    if (ConsolePlayerNumber() == pln)
    {
        if (!Client_GotPickup[myPln][index])
        {
            if ((StrLen(message) != 0) && (GetCvar("msg") <= 0))
            {
                int col = msgColors[GetCVar("msg0color")];
                Log(s:col, s:strsub(message, "\c-", col));
            }

            if (silent)
            {
                LocalAmbientSound(pkupSound, 127);
            }
        }

        int r = DakkaPickupFlashes[classslot][index][0];
        int g = DakkaPickupFlashes[classslot][index][1];
        int b = DakkaPickupFlashes[classslot][index][2];
        int i = DakkaPickupFlashes[classslot][index][3];
        int t = DakkaPickupFlashes[classslot][index][4];

        FadeRange(r, g, b, i, r, g, b, 0, t);
    }

    if (!(silent || Client_GotPickup[myPln][index]))
    {
        ActivatorSound(pkupSound, 127);
    }

    Client_GotPickup[myPln][index] = 1;
}

script 646 (void) clientside
{
    int class = Client_Class + 1;
    int state = DakkaDisplayStates[class];

    if (ConsolePlayerNumber() == -1)
    {
        SetActorState(0, "ServerDoNothing");
    }
    else
    {
        SetActorState(0, state);
    }

    int wait = 0;
    while (Timer() < 1)
    {
        wait++;
        Delay(1);
    }

    if (wait) { ACS_ExecuteWithResult(646); }
}
