function int CalcPointValue(void)
{
    return GetActorProperty(0, APROP_SpawnHealth);
}

function void DrawScore(int curPoints)
{
    SetFont("BIGFONT");
    SetHudSize(480, 360, 1);
    HudMessage(s:"Score: \cf", d:curPoints;
                HUDMSG_FADEOUT, 24200, CR_WHITE, 340.1, 32.1, 0.5, 0.25);
}

int Tmp_FireModesUsed[SCOREFIREMODES];

function int WeaponSwitchMultiplier(int pln, int myhp)
{
    if (pln < 0 || pln >= PLAYERMAX) { return 0.0; }

    int maxtics = round(SWITCHAROO_TICSPER100HP * myhp * 0.01);
    maxtics = middle(SWITCHAROO_MINTICS, maxtics, SWITCHAROO_MAXTICS);
    int cutoff = Timer() - maxtics;
    
    int wepsUsed        = 0;
    int fireModesUsed   = 0;
    int s, f, i;
    int used;

    for (i = 0; i < SCOREFIREMODES; i++)
    {
        Tmp_FireModesUsed[i] = 0;
    }

    for (s = 0; s < SCOREWEPS; s++)
    {
        used  = 0;

        for (f = 0; f < SCOREFIREMODES; f++)
        {
            i = PlayerWeapons_LastFired[pln][s][f];
            if (i == -1) { continue; }

            if (cutoff <= i)
            {
                used = 1;
                Tmp_FireModesUsed[f] = 1;
            }
        }

        if (used) { wepsUsed += 1; }
    }

    for (i = 0; i < SCOREFIREMODES; i++)
    {
        if (Tmp_FireModesUsed[i]) { fireModesUsed++; }
    }

    if (wepsUsed > 1) { return 0.15; }
    if (fireModesUsed > 1) { return 0.075; }
    return 0.0;
}

function int KillstreakMultiplier(int pln)
{
    if (pln < 0 || pln >= PLAYERMAX) { return 0.0; }

    int curtime = Timer();
    int neededTime = PlayerKillStreaks[pln][KS_TIME] + PlayerKillStreaks[pln][KS_TIMEADD];

    if (curtime > neededTime)
    {
        PlayerKillStreaks[pln][KS_COUNT] = 0;
        return 0.0;
    }

    return min(KILLSTREAK_MULTMAX, KILLSTREAK_MULTINC * PlayerKillStreaks[pln][KS_COUNT]);
}

function int AddKillstreak(int pln, int myhp)
{
    if (pln < 0 || pln >= PLAYERMAX) { return 0; }

    int timeAdd = round(KILLSTREAK_TICSPER100HP * myhp * 0.01);
    timeAdd = middle(KILLSTREAK_MINTICS, timeAdd, KILLSTREAK_MAXTICS);

    PlayerKillStreaks[pln][KS_COUNT]   += 1;
    PlayerKillStreaks[pln][KS_TIME]     = Timer();
    PlayerKillStreaks[pln][KS_TIMEADD]  = timeAdd;
    return 0;
}

function void DisplayBonusValues(int pln)
{
    if (pln < 0 || pln >= PLAYERMAX) { return; }
    SetHudSize(640, 480, 1);

    int i, score, name, color, offset = 0;

    for (i = 0; i < BONUSCOUNT; i++)
    {
        name  = BonusNames[i];
        color = BonusColors[i];
        score = BonusValues[pln][i];
        if (score <= 0) { continue; }
        if (Timer() > LastBonus[pln][i]) { continue; }

        HudMessage(s:"+", d:score; HUDMSG_FADEOUT, 24201 + (offset * 2), color, 
                        455.1, 72.0 + (13.0 * offset), 0.5, 0.5);

        HudMessage(s:name; HUDMSG_FADEOUT, 24202 + (offset * 2), CR_WHITE, 
                        508.1, 72.0 + (13.0 * offset), 0.5, 0.5);

        offset += 1;
    }
}

function int BoneDryMultiplier(void)
{
    int hp = GetActorProperty(0, APROP_Health);
    if (hp < 0) { return 0; }

    int hpUnder = max(0, (BONEDRY_MAXHP - hp) + 1);

    return BONEDRY_MULTPERHP * hpUnder;
}

function int SoreLoserMultiplier(void)
{
    int hp = GetActorProperty(0, APROP_Health);

    if (hp < 0) { return 2.0; }
    return 0;
}

function int BrawlerMultiplier(void)
{
    int i, wep, mult;

    for (i = 0; i < KNOWNMELEE; i++)
    {
        wep  = Brawler_KnownMelee[i];
        mult = Brawler_MeleeMults[i];
        if (CheckWeapon(wep)) { return mult; }
    }

    return 0;
}

function int AirMultiplier(int pln)
{
    if (pln < 0 || pln >= PLAYERMAX) { return 0; }

    int zvel  = PlayerZHEIGHT[pln];
    int zdiff = max(0, zvel - AIR_MINHEIGHT);

    return min(AIR_MAXMULT, FixedMul(zdiff, AIR_MULT));
}

function int CurveballMultiplier(int couldsee)
{
    int ret;

    if (couldsee) { ret = 0; }
    else { ret = CURVEBALL_MULT; }
    
    return ret;
}

function int ScrappingMultiplier(int pln)
{
    if (pln < 0 || pln >= PLAYERMAX) { return 0.0; }
    
    int s, f, i;

    int latestTic  = -1;
    int latestWep  = -1;
    int latestMode = -1;

    for (i = 0; i < SCOREFIREMODES; i++)
    {
        Tmp_FireModesUsed[i] = 0;
    }

    for (s = 0; s < SCOREWEPS; s++)
    {
        for (f = 0; f < SCOREFIREMODES; f++)
        {
            i = PlayerWeapons_LastFired[pln][s][f];
            
            if (i > latestTic)
            {
                latestTic  = i;
                latestWep  = s;
                latestMode = f;
            }
        }
    }

    if (latestWep == S_SCRAPPER) { return SCRAPPING_MULT; }
    return 0;
}

// Point multiplier ideas:
//
// EFFICIENCY:
// - Spree: the more dudes you kill in quick succession, the more points you get.
//
// STYLE:
// - Switcharoo: switch up your weapons and fire modes when fighting? get more points!
// - Air: kill enemies mid-flight, get more points
// - Action hero: be facing away from the enemy when you kill them, get points!
// - Ghost: kill an enemy without them being able to see you? get points.
// - Scrapping: scrapper kill
//
// DARING:
// - Fisticuffs: use your fists to get kills
// - Bone-dry: kill enemies with no more than 25 hp
// - Sore loser: kill enemies while dead (overrides bone-dry)
//

int TmpBonuses[BONUSCOUNT];

script 476 (int pointValue)
{
    int myhp = GetActorProperty(0, APROP_SpawnHealth);
    int i;
    if (pointValue == -1) { pointValue = CalcPointValue(); }

    int plZ = ACS_ExecuteWithResult(648, 10, 2);
    int myZ = GetActorZ(0);
    int playerLower = plZ < myZ;

    SetActorPosition(0, GetActorX(0), GetActorY(0), myZ+41.0, 0);
    GiveInventory("DakkaCurveballChecker", 1);
    int curveCheck = CheckInventory("DakkaCurveballToken");
    SetActorPosition(0, GetActorX(0), GetActorY(0), myZ, 0);

    SetActivatorToTarget(0);

    int pln = PlayerNumber();
    if (pln == -1) { terminate; }
    if (CtoS_NoScore[pln]) { terminate; }

    int points_switcharoo   = round(pointValue * WeaponSwitchMultiplier(pln, myhp));
    int points_killstreak   = round(pointValue * KillstreakMultiplier(pln));
    int points_bonedry      = round(pointValue * BoneDryMultiplier());
    int points_soreloser    = round(pointValue * SoreLoserMultiplier());
    int points_brawler      = round(pointValue * BrawlerMultiplier());
    int points_air          = round(pointValue * AirMultiplier(pln));
    int points_curveball    = round(pointValue * CurveballMultiplier(curveCheck));
    int points_scrapping    = round(pointValue * ScrappingMultiplier(pln));

    PlayerMapScores[pln] += pointValue;
    PlayerMapScores[pln] += points_switcharoo;
    PlayerMapScores[pln] += points_killstreak;
    PlayerMapScores[pln] += points_bonedry;
    PlayerMapScores[pln] += points_soreloser;
    PlayerMapScores[pln] += points_brawler;
    PlayerMapScores[pln] += points_air;
    PlayerMapScores[pln] += points_curveball;
    PlayerMapScores[pln] += points_scrapping;

    int bonustime = Timer();

    TmpBonuses[BS_BASE]         = pointValue;
    TmpBonuses[BS_SPREE]        = points_killstreak;
    TmpBonuses[BS_SWITCHAROO]   = points_switcharoo;
    TmpBonuses[BS_BONEDRY]      = points_bonedry;
    TmpBonuses[BS_SORELOSER]    = points_soreloser;
    TmpBonuses[BS_BRAWLER]      = points_brawler;
    TmpBonuses[BS_AIR]          = points_air;
    TmpBonuses[BS_CURVEBALL]    = points_curveball;
    TmpBonuses[BS_SCRAPPING]    = points_scrapping;

    for (i = 0; i < BONUSCOUNT; i++)
    {
        if (TmpBonuses[i] > 0)
        {
            BonusValues[pln][i] += TmpBonuses[i];
            LastBonus[pln][i] = bonustime;
        }
    }

    DisplayBonusValues(pln);

    AddKillstreak(pln, myhp);
    Delay(28);

    for (i = 0; i < BONUSCOUNT; i++)
    {
        if (LastBonus[pln][i] == bonustime)
        {
            BonusValues[pln][i] = 0;
        }
    }
}

script 397 (int slot, int firemode)
{
    int pln = PlayerNumber();
    if (pln == -1) // called from projectile
    {
        SetActivatorToTarget(0);
        pln = PlayerNumber();
    }

    if (pln == -1
     || slot < 0 || slot >= SCOREWEPS
     || firemode < 0 || firemode >= SCOREFIREMODES) { terminate; }

    PlayerWeapons_LastFired[pln][slot][firemode] = Timer();
}
